/**
 * Illustrates advanced (branching) recursive method implementations,
 * to generate combinatorial objects (bit patterns).
 */
public class BitPatterns {
    
/**
 * @pre   true
 * @post  each 3-bit pattern has been printed exactly once
 */
public static void generateAll3BitPatterns() { // Not recursive
    for (int b2 = 0; b2 <= 1; ++ b2) {
        for (int b1 = 0; b1 <= 1; ++ b1) {
            for (int b0 = 0; b0 <= 1; ++ b0) {
                System.out.println ( "" + b2 + b1 + b0 );
             }
         }
    }
}

/**
 * @pre   0 <= n
 * @post  each bit pattern with prefix s and n additional bits
 *   (that is, s.length()+n bits) has been printed exactly once
 * @bound n
 */
public static void generate(String s, int n) {
    if (n == 0) {  // base case
        System.out.println(s);
    } else {  // 0 < n, inductive step
        for (int b = 0; b <= 1; ++ b) {
            // pre: 0 <= n - 1; bound: n - 1 < n
            generate(s + b, n - 1);
        }
    }
}

/**
 * @pre   0 <= n && 0 <= k && m == (\num_of i; s.has(i); s.charAt(i) == '1')
 * @post  each bit pattern with prefix s and n additional bits,
 *   having k 1's, has been printed exactly once
 * @bound n
 */
public static void generate(String s, int n, int k, int m) { // Inefficient
    if (n == 0) {  // base case: all bits are there
        if (m == k) {  // keep the desired patterns
            System.out.println(s);
        } else { /* m != k: drop the others */ }
    } else {  // 0 < n, inductive step: extend by one bit b
        for (int b = 0; b <= 1; ++ b) {
            generate(s + b, n - 1, k, m + b);
        }
    }
}

/**
 * @pre   0 <= k <= n
 * @post  each bit pattern with prefix s and n additional bits,
 *   of which k 1's, has been printed exactly once
 * @bound n
 */
public static void generateA(String s, int n, int k) {
    if (n == 0) {  // k == 0, base case
        System.out.println(s);
    } else {  // 0 < n, inductive step
        if (k <= n - 1) {  // can afford a 0
            generate(s + 0, n - 1, k);
        }
        if (1 <= k) {  // can afford a 1
            generate(s + 1, n - 1, k - 1);
        }
    }
}

/**
 * @pre   0 <= k <= n
 * @post  each bit pattern with prefix s and n additional bits,
 *   of which k 1's, has been printed exactly once
 * @bound n
 */
public static void generate(String s, int n, int k) {
    if (n == 0) {  // k == 0, base case
        System.out.println(s);
    } else {  // 0 < n, inductive step
        for (int b = 0; b <= 1; ++ b) {
            if (0 <= k - b && k - b <= n - 1) {
                generate(s + b, n - 1, k - b);
            }
        }
    }
}

}