import java.util.List;
import java.util.ArrayList;

/**
 * Generator for combinatorial objects of n bits with k 1s,
 * with multiple listeners.
 */
public class ObservableGenerator {
    
    /** The registered listeners */
    private final List<GeneratorListener> listeners;
    
    /**
     * Constructs a generator without listeners.
     */
    public ObservableGenerator() {
        this.listeners = new ArrayList<GeneratorListener>();
    }
    
    /**
     * Adds a listener for generator events.
     */
    public void addListener(GeneratorListener listener) {
        listeners.add(listener);
    }

    /**
     * Removes a listener for generator events.
     */
    public void removeListener(GeneratorListener listener) {
        listeners.remove(listener);
    }

    /**
     * Notifies all registered listeners.
     */
    void notifyListeners(String s) {
        for (GeneratorListener listener : listeners) {
            listener.objectGenerated(s);
        }
    }
    
    /**
     * @pre   0 <= k <= n
     * @post  each n-bit pattern with  k 1's
     *   has been reported to listeners exactly once
     */
    public void generate(int n, int k) {
        generate("", n, k);
    }
    
    /**
     * @pre   0 <= k <= n
     * @post  each bit pattern with prefix s and n additional bits,
     *   of which k 1's, has been reported to listeners exactly once
     * @bound n
     */
    void generate(String s, int n, int k) {
        if (n == 0) {  // k == 0, base case
            notifyListeners(s);
        } else {  // 0 < n, inductive step
            for (int b = 0; b <= 1; ++b) {
                if (0 <= k - b && k - b <= n - 1) {
                    generate(s + b, n - 1, k - b);
                }
            }
        }
    }

}